from django.conf import settings
from django.contrib import messages
from django.contrib.auth import get_user_model, user_logged_in
from django.db.models import signals
from django.utils import timezone

from .models import PasswordChange, PasswordHistory
from .utils import PasswordChecker


def create_user_handler(sender, instance, created, **kwargs):
    # when the user is created, set the password last changed field to now.
    # Create the new row in PasswordHistory and delete the old one if necessary.
    if created:
        now = timezone.now()
        PasswordChange.objects.create(user=instance, last_changed=now)
        PasswordHistory.objects.create(user=instance, created=now, password=instance.password)
        PasswordHistory.objects.delete_expired(instance)


def change_password_handler(sender, instance, **kwargs):
    # Checks if the user changed password
    # contrib/auth/base_user.py sets _password in set_password()
    if instance._password is None:
        return

    try:
        get_user_model().objects.get(id=instance.id)
    except get_user_model().DoesNotExist:
        return

    # We update the PasswordChange, create a new row in PasswordHistory and delete an old row if necessary
    now = timezone.now()
    record, _ = PasswordChange.objects.get_or_create(user=instance)
    record.last_changed = now
    record.save()

    # NOTE When changing the password, `set_password` is called 2 times: 1 time when the
    # the form in ForcePasswordChangeView is saved and another time after this view.
    # We allow only the 1st storage of the password.
    if getattr(instance, '_has_not_previous_password', False):
        PasswordHistory.objects.create(user=instance, created=now, password=instance.password)
        PasswordHistory.objects.delete_expired(instance)
        instance._has_not_previous_password = False


def login_handler(sender, request, user, **kwargs):
    checker = PasswordChecker(user)
    if checker.is_expired():
        # Login with expired password then redirect to change the password.
        # This solution is faster and probably as safe as resetting the password
        # by sending a token the user by email.
        messages.error(request, "Password expired. You have to change your password.")

        # set flag for middleware to pick up
        request.redirect_to_password_change = True


def register_signals():
    signals.post_save.connect(
        create_user_handler,
        sender=settings.AUTH_USER_MODEL,
        dispatch_uid="password_rotate:create_user_handler",
    )

    signals.pre_save.connect(
        change_password_handler,
        sender=settings.AUTH_USER_MODEL,
        dispatch_uid="password_rotate:change_password_handler",
    )

    user_logged_in.connect(
        login_handler,
        dispatch_uid="password_rotate:login_handler"
    )



from django.contrib import messages
from django.shortcuts import redirect
from django.urls import resolve, reverse
from django.utils.safestring import mark_safe

from .utils import PasswordChecker, request_is_ajax


class PasswordRotateMiddleware:
    """
    Adds Django message if password expires soon.
    Checks if user should be redirected to change password.
    """
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        request.password_status = "valid"
        if self.is_page_for_warning(request):
            force_password_change_path = reverse("force_password_change")
            # add warning if within the notification window for password expiration
            if request.user.is_authenticated:
                checker = PasswordChecker(request.user)
                msg = f"<a href='{force_password_change_path}'>Please change your password.</a> "
                if checker.is_expired():
                    if request.path != force_password_change_path:
                        msg += "It has expired."
                        self.add_warning(request, mark_safe(msg))
                        request.password_status = "expired"
                else:
                    time_to_expire_string = checker.get_expire_time()
                    request.password_status = "valid"
                    if time_to_expire_string and request.path != force_password_change_path:
                        msg += f"It expires in {time_to_expire_string}."
                        self.add_warning(request, mark_safe(msg))

        # picks up flag for forcing password change
        if hasattr(request, "redirect_to_password_change"):
            return redirect("force_password_change")

        # At this point, if the password expired, the user should have been redirected to force_password
        # change and should have changed her password.
        # If the user didn't change her password, redirect her until it's done.
        # If the password is still valid, just return the response.
        if request.password_status == "valid":
            return self.get_response(request)
        elif request.password_status == "expired":
            return redirect("force_password_change")

    def is_page_for_warning(self, request):
        """
        Only warn on pages that are GET requests and not ajax. Also ignore logouts.
        """
        if request.method == "GET" and not request_is_ajax(request):
            match = resolve(request.path)
            if match and match.url_name == "logout":
                return False
            return True
        return False

    def add_warning(self, request, text):
        storage = messages.get_messages(request)
        for message in storage:
            # only add this message once
            if message.extra_tags is not None and "password_rotate" in message.extra_tags:
                return
        messages.warning(request, text, extra_tags="password_rotate")



from django.contrib.auth.forms import PasswordChangeForm
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

from rapidfuzz import fuzz
from django.conf import settings


class ForcePasswordChangeForm(PasswordChangeForm):
    """
    This form ensures the new password is not too similar to the old password.
    """
    def clean(self):
        cleaned_data = super().clean()

        if cleaned_data.get("old_password") and cleaned_data.get("new_password1"):
            ratio = fuzz.ratio(
                cleaned_data["old_password"],
                cleaned_data["new_password1"]
            )
            if ratio >= settings.PASSWORD_ROTATE_MAX_SIMILARITY_RATIO:
                raise ValidationError(
                    {"new_password1": _("The new password is too similar to the old one.")},
                    code="password_similar"
                )
        return cleaned_data



from datetime import timedelta

from django.conf import settings
from django.utils import timezone
import humanize

from .models import PasswordChange


def request_is_ajax(request):
    """replace deprecated `request.is_ajax` method"""
    return request.META.get("HTTP_X_REQUESTED_WITH") == "XMLHttpRequest"


class PasswordChecker:
    """
    Checks if password has expired or if it will expire soon
    """
    def __init__(self, user):
        # password expires: last_changed + password_duration
        self.password_allowed_duration = timedelta(seconds=settings.PASSWORD_ROTATE_SECONDS)
        # start warning at password expiration - duration
        self.password_warning_duration = timedelta(seconds=settings.PASSWORD_ROTATE_WARN_SECONDS)

        self.user = user
        self.last_changed = self.get_last_changed()
        self.expiration = self.last_changed + self.password_allowed_duration
        self.warning = self.expiration - self.password_warning_duration

    def is_expired(self):
        if self.is_user_excluded():
            return False
        return timezone.now() > self.expiration

    def is_warning(self):
        if self.is_user_excluded():
            return False
        return timezone.now() > self.warning

    def get_expire_time(self):
        """
        Gets the expiration time as string if within the warning duration.
        Otherwise, returns None.
        """
        if self.is_warning():
            time_left = self.expiration - timezone.now()
            return humanize.naturaldelta(time_left)
        else:
            return None

    def get_last_changed(self):
        # if no record, fallback to when user created
        try:
            record = PasswordChange.objects.get(user=self.user)
            last_changed = record.last_changed
        except PasswordChange.DoesNotExist:
            last_changed = self.user.date_joined
        return last_changed

    def is_user_excluded(self):
        # admin can configure so superusers are excluded from check
        if hasattr(settings, "PASSWORD_ROTATE_EXCLUDE_SUPERUSERS") and\
                settings.PASSWORD_ROTATE_EXCLUDE_SUPERUSERS:
            return self.user.is_superuser
        return False



from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _
from password_rotate.managers import PasswordHistoryManager


class PasswordChange(models.Model):
    """
    Records when users change a password to support an expiration policy
    """
    last_changed = models.DateTimeField(db_index=True, auto_now_add=True)
    user = models.OneToOneField(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)

    def __str__(self):
        return f"{self.user.username}"


class PasswordHistory(models.Model):
    """
    Stores a single password history entry, related to :model:`auth.User`.
    """
    created = models.DateTimeField(
        auto_now_add=True, verbose_name=_("created"), db_index=True, help_text=_("The date the entry was created.")
    )
    password = models.CharField(
        max_length=128, verbose_name=_("password"), help_text=_("The encrypted password.")
    )
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        verbose_name=_("user"),
        help_text=_("The user this password history entry belongs to."),
        related_name="password_history_entries",
        on_delete=models.CASCADE,
    )

    objects = PasswordHistoryManager()

    def __str__(self):
        return f"{self.user.username} - {self.created}"

    class Meta:
        get_latest_by = "created"
        ordering = ["-created"]
        verbose_name = _("password history entry")
        verbose_name_plural = _("password history entries")



from django.contrib.auth import update_session_auth_hash
from django.contrib.auth.views import PasswordChangeView

from password_rotate.forms import ForcePasswordChangeForm


class ForcePasswordChangeView(PasswordChangeView):
    """
    This view forces the user to change her password when it expired.

    Updating the password properly will change the password status from "expired"
    to "valid".
    """
    form_class = ForcePasswordChangeForm

    def form_valid(self, form):
        form.save()
        # Updating the password logs out all other sessions for the user
        # except the current one.
        update_session_auth_hash(self.request, form.user)
        self.request.password_status = "valid"
        return super().form_valid(form)



# Generated by Django 2.2.27 on 2022-03-26 12:46

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="PasswordChange",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("last_changed", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("user", models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name="PasswordHistory",
            fields=[
                ("id", models.BigAutoField(verbose_name="ID", serialize=False, auto_created=True, primary_key=True)),
                ("created", models.DateTimeField(auto_now_add=True, help_text="The date the entry was created.", verbose_name="created", db_index=True)),
                ("password", models.CharField(help_text="The encrypted password.", max_length=128, verbose_name="password")),
                ("user", models.ForeignKey(related_name="password_history_entries", verbose_name="user", to=settings.AUTH_USER_MODEL, help_text="The user this password history entry belongs to.", on_delete=django.db.models.deletion.CASCADE)),
            ],
            options={
                "ordering": ["-created"],
                "get_latest_by": "created",
                "verbose_name": "password history entry",
                "verbose_name_plural": "password history entries",
            },
            bases=(models.Model,),
        ),

    ]



from datetime import timedelta
from unittest import mock

from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.hashers import identify_hasher
from django.test import TestCase
from django.utils import timezone
from django.urls import reverse

from password_rotate.models import PasswordChange, PasswordHistory
from password_rotate.utils import PasswordChecker


def do_nothing(*args, **kwargs):
    pass


def create_user(username="bob", password="password", date_joined=timezone.now()):
    user = get_user_model()(
        username=username,
        email=f"{username}@example.org",
        is_active=True,
        date_joined=date_joined,
    )
    user.set_password(password)
    user.save()
    return user


class BaseTestCase(TestCase):
    def setUp(self):
        PasswordChange.objects.all().delete()
        PasswordHistory.objects.all().delete()

    def force_password_change(self, old, new):
        return self.client.post(
            reverse("force_password_change"),
            {
                "old_password": old, "new_password1": new, "new_password2": new,
            }
        )


class PasswordCheckerTests(BaseTestCase):
    def test_rotate_with_time_to_go_using_password_change_model(self):
        user = create_user(date_joined=timezone.now())
        checker = PasswordChecker(user)
        self.assertFalse(checker.is_expired())

    def test_rotate_with_time_to_go_using_date_joined(self):
        user = create_user(date_joined=timezone.now())
        PasswordChange.objects.all().delete()
        checker = PasswordChecker(user)
        self.assertFalse(checker.is_expired())

    def test_rotate_with_no_time_to_go_using_password_change_model(self):
        user = create_user(date_joined=timezone.now())
        record = PasswordChange.objects.get(user=user)
        record.last_changed = timezone.now() - timedelta(seconds=settings.PASSWORD_ROTATE_SECONDS + 1)
        record.save()
        checker = PasswordChecker(user)
        self.assertTrue(checker.is_expired())

    def test_rotate_with_no_time_to_go_using_date_joined(self):
        join_date = timezone.now() - timedelta(seconds=settings.PASSWORD_ROTATE_SECONDS + 1)
        user = create_user(date_joined=join_date)
        PasswordChange.objects.all().delete()
        checker = PasswordChecker(user)
        self.assertTrue(checker.is_expired())

    def test_warn_with_time_to_go_using_password_change_model(self):
        user = create_user(date_joined=timezone.now())
        checker = PasswordChecker(user)
        self.assertFalse(checker.is_warning())

    def test_warn_with_time_to_go_using_date_joined(self):
        user = create_user(date_joined=timezone.now())
        PasswordChange.objects.all().delete()
        checker = PasswordChecker(user)
        self.assertFalse(checker.is_warning())

    def test_warn_with_no_time_to_go_using_password_change_model(self):
        user = create_user(date_joined=timezone.now())
        record = PasswordChange.objects.get(user=user)
        record.last_changed = timezone.now() - timedelta(seconds=settings.PASSWORD_ROTATE_SECONDS - 1)
        record.save()
        checker = PasswordChecker(user)
        self.assertTrue(checker.is_warning())
        self.assertFalse(checker.is_expired())

    def test_warn_with_no_time_to_go_using_date_joined(self):
        join_date = timezone.now() - timedelta(seconds=settings.PASSWORD_ROTATE_SECONDS - 1)
        user = create_user(date_joined=join_date)
        PasswordChange.objects.all().delete()
        checker = PasswordChecker(user)
        self.assertTrue(checker.is_warning())
        self.assertFalse(checker.is_expired())


class PasswordValidatorsTests(BaseTestCase):
    def test_password_never_used(self):
        """
        Updating the password with a password never used should be allowed
        """
        # ARRANGE
        credentials = {"username": "bob", "password": "password"}
        user = create_user(**credentials)
        self.client.login(**credentials)

        # ACT
        response = self.force_password_change("password", "some new words")

        # ASSERT
        # There should be no error message
        self.assertNotContains(
            response, "You already used this password.", status_code=302
        )

        self.assertEqual(PasswordChange.objects.all().count(), 1)
        # There should be 2 rows: password and password1
        self.assertEqual(PasswordHistory.objects.filter(user=user).count(), 2)

    def test_password_already_used(self):
        """
        Updating the password with a password already used should not be allowed
        """
        # ARRANGE
        credentials = {"username": "bob", "password": "password"}
        user = create_user(**credentials)
        self.client.login(**credentials)

        # ACT
        # 1st password change, we use a new password
        response = self.force_password_change("password", "some new words")
        pwcs = PasswordChange.objects.all()
        # 2nd password change, we use the initial password
        response = self.force_password_change("some new words", "password")

        # ASSERT
        # The user should be warned with an error message
        self.assertContains(
            response, "You already used this password.", status_code=200
        )

        # The password changed only the first time
        self.assertEqual(PasswordChange.objects.all().count(), 1)
        self.assertEqual(PasswordChange.objects.all()[0], pwcs[0])

        # There should be only 2 rows although 3 passwords where typed.
        # The last one is refused because it doesn't validate the
        # `password_rotate.validators.NotPreviousPasswordValidator`
        self.assertEqual(PasswordHistory.objects.filter(user=user).count(), 2)


class PasswordHistoryCountTest(BaseTestCase):
    def test_history_count(self):
        """
        The passwords stored in `PasswordHistory` should be limited
        to the latest `PASSWORD_ROTATE_HISTORY_COUNT`
        """
        # ARRANGE
        credentials = {"username": "bob", "password": "password"}
        create_user(**credentials)
        self.client.login(**credentials)

        raw_passwords = [
            "password",
            "Hello world",
            "Bonjour le monde",
            "Goodbye world",
            "Au revoir django",
        ]

        # ACT
        # We change the password PASSWORD_ROTATE_HISTORY_COUNT + 1 times
        for i in range(settings.PASSWORD_ROTATE_HISTORY_COUNT + 1):
            response = self.force_password_change(raw_passwords[i], raw_passwords[i+1])

        # ASSERT
        self.assertNotContains(response, "You already used this password.", status_code=302)
        self.assertEqual(PasswordChange.objects.count(), 1)
        self.assertEqual(PasswordHistory.objects.count(), settings.PASSWORD_ROTATE_HISTORY_COUNT)

        # Only the N latest passwords should be stored (N = PASSWORD_ROTATE_HISTORY_COUNT)
        encrypted_passwords = (
            PasswordHistory.objects.values_list("password", flat=True)
            .order_by("created")
        )
        # The first 2 passwords should be removed from PasswordHistory
        for raw, encrypted in zip(raw_passwords[2:], encrypted_passwords):
            hasher = identify_hasher(encrypted)
            # NOTE A password is verified if encrypt(raw) == encrypted
            self.assertTrue(hasher.verify(raw, encrypted))


class ForcePasswordChangeTests(BaseTestCase):
    @mock.patch("password_rotate.signals.messages", side_effect=do_nothing())
    def test_redirection_while_password_not_changed(self, messages):
        """
        When the password expired, while the user didn't change it, she should
        be redirected to the "force_password_change" url pattern.
        """
        # ARRANGE
        # The password expired
        user = create_user(date_joined=timezone.now())
        record = PasswordChange.objects.get(user=user)
        record.last_changed = timezone.now() - timedelta(seconds=settings.PASSWORD_ROTATE_SECONDS + 1)
        record.save()

        fpc_url = reverse("force_password_change")

        credentials = {"username": "bob", "password": "password"}
        self.client.login(**credentials)

        # ACT
        responses = {}
        responses["/some_page/"] = self.client.get("/some_page/")
        responses["/admin/"] = self.client.get("/admin/")
        responses[fpc_url] = self.client.get(fpc_url)

        # ASSERT
        self.assertEqual(responses["/some_page/"].status_code, 302)
        self.assertEqual(responses["/some_page/"]["Location"], fpc_url)
        self.assertEqual(responses["/admin/"].status_code, 302)
        self.assertEqual(responses["/admin/"]["Location"], fpc_url)
        self.assertEqual(responses[fpc_url].status_code, 200)

    @mock.patch("password_rotate.signals.messages", side_effect=do_nothing())
    def test_redirection_stops_after_password_change(self, messages):
        """
        When the password expired, while the user didn't change it, she should
        be redirected to the "force_password_change" url pattern.
        """
        # ARRANGE
        # The password expired
        user = create_user(date_joined=timezone.now())
        record = PasswordChange.objects.get(user=user)
        record.last_changed = timezone.now() - timedelta(seconds=settings.PASSWORD_ROTATE_SECONDS + 1)
        record.save()
        credentials = {"username": "bob", "password": "password"}
        self.client.login(**credentials)

        # ACT
        self.force_password_change("password", "some new words")

        # ASSERT
        self.assertEqual(self.client.get("/some_page/").status_code, 200)


class PasswordSimilarityRatioTest(BaseTestCase):
    def test_password_with_high_similarity(self):
        """
        When the new password is too similar to the old one, it should not be validated.
        """
        # ARRANGE
        user = create_user(date_joined=timezone.now())
        credentials = {"username": "bob", "password": "password"}
        self.client.login(**credentials)
        pwcs = PasswordChange.objects.all()

        # ACT
        # fuzz.ratio("password", "password1") gives a similarity of 94%
        # which is higher than the max similarity we set (30 %)
        response = self.force_password_change("password", "password1")

        # ASSERT
        # An error message. We should stay on this page.
        self.assertContains(
            response, "The new password is too similar to the old one.", status_code=200
        )

        # The password didn't change
        self.assertEqual(PasswordChange.objects.all().count(), 1)
        self.assertEqual(PasswordChange.objects.all()[0], pwcs[0])

        # There should be only 1 row
        self.assertEqual(PasswordHistory.objects.filter(user=user).count(), 1)

    def test_password_with_low_similarity(self):
        """
        When the similarity of the new password is low, it should be validated.
        """
        # ARRANGE
        user = create_user(date_joined=timezone.now())
        pwc = PasswordChange.objects.all()[0]
        credentials = {"username": "bob", "password": "password"}
        self.client.login(**credentials)

        # ACT
        # fuzz.ratio("password", "some new words")
        # gives a similarity of 45% which is lower than the max similarity we set (50 %)
        response = self.force_password_change("password", "some new words")

        # ASSERT
        # No error message and we should be redirected to password_change_done
        self.assertNotContains(
            response, "The new password is too similar to the old one.", status_code=302
        )
        self.assertEqual(response["Location"], reverse("password_change_done"))

        # The password changed
        self.assertEqual(PasswordChange.objects.all().count(), 1)
        self.assertGreater(PasswordChange.objects.all()[0].last_changed, pwc.last_changed)

        # There should be 2 rows
        self.assertEqual(PasswordHistory.objects.filter(user=user).count(), 2)



from django.apps import AppConfig


class PasswordRotateConfig(AppConfig):
    name = 'password_rotate'
    default_auto_field = "django.db.models.BigAutoField"

    def ready(self):
        from . import signals
        signals.register_signals()



from django.utils.translation import gettext_lazy as _
from django.core.exceptions import ValidationError

from password_rotate.models import PasswordHistory


class NotPreviousPasswordValidator:
    """
    Validate that the password is not a previously used password.

    The password is rejected if it exists in `PasswordHistory`.
    """

    def validate(self, password, user=None):
        """
        Return True when the password has not been stored in `PasswordHistory`
        """
        if not PasswordHistory.objects.check_password(user, password):
            raise ValidationError("You already used this password.")
        else:
            # This flag allows to store the password in `PasswordHistory`
            # when `set_password` is called
            user._has_not_previous_password = True
            return True

    def get_help_text(self):
        return _("Your password must be different from any previous one.")



from django.urls import include, path
from django.contrib import admin
from django.http import HttpResponse


def some_page(request):
    return HttpResponse("Welcome to some page!")


urlpatterns = [
    path("", include("django.contrib.auth.urls")),
    path("admin/", admin.site.urls),
    path("password_rotate/", include("password_rotate.urls")),
    path("some_page/", some_page),
]



import os
from django.contrib.admin import templates


DEBUG = False
LANGUAGES = (("en", "English"),)
LANGUAGE_CODE = "en"
USE_TZ = False
USE_I18N = True
SECRET_KEY = "fake-key"
DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'

PASSWORD_ROTATE_SECONDS = 10 * 60  # 10 minutes
PASSWORD_ROTATE_WARN_SECONDS = 5 * 60  # 5 minutes
PASSWORD_ROTATE_HISTORY_COUNT = 3
PASSWORD_ROTATE_MAX_SIMILARITY_RATIO = 50

INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.admin",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.messages",
    "password_rotate",
]

MIDDLEWARE = (
    "django.middleware.common.CommonMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "password_rotate.middleware.PasswordRotateMiddleware",
)

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
        "TEST_NAME": ":memory:",
        "USER": "",
        "PASSWORD": "",
        "PORT": "",
    },
}

AUTH_PASSWORD_VALIDATORS = [
    {
     "NAME": "password_rotate.validators.NotPreviousPasswordValidator",
    },
]

ROOT_URLCONF = "password_rotate.tests.urls"

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [templates.__path__[0]],
        'APP_DIRS': False,
        "OPTIONS": {
            "debug": DEBUG,
            "loaders": [
                (
                    "django.template.loaders.cached.Loader",
                    [
                        "django.template.loaders.filesystem.Loader",
                        "django.template.loaders.app_directories.Loader",
                    ],
                )
            ],
            "context_processors": (
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages",
                "django.template.context_processors.request",
            ),
        }
    }
]



from django.contrib import admin
from password_rotate.models import PasswordChange, PasswordHistory


@admin.register(PasswordChange)
class PasswordChangeAdmin(admin.ModelAdmin):
    model = PasswordChange
    list_display = ("user", "last_changed")


@admin.register(PasswordHistory)
class PasswordHistory(admin.ModelAdmin):
    model = PasswordHistory
    list_display = ("user", "created")



from django.db import models
from django.contrib.auth.hashers import identify_hasher
from django.conf import settings


class PasswordHistoryManager(models.Manager):
    default_offset = settings.PASSWORD_ROTATE_HISTORY_COUNT

    def delete_expired(self, user, offset=None):
        """
        Deletes expired password history entries from the database(s).

        :arg user: A :class:`~django.contrib.auth.models.User` instance.
        :arg int offset: A number specifying how much entries are to be kept
              in the user's password history. Defaults
              to :py:attr:`~settings.PASSWORD_ROTATE_HISTORY_COUNT`.
        """
        if not offset:
            offset = self.default_offset
        qs = self.filter(user=user)
        if qs.count() > offset:
            entry = qs[offset:offset + 1].get()
            qs.filter(created__lte=entry.created).delete()

    def check_password(self, user, raw_password):
        """
        Compares a raw (UNENCRYPTED!!!) password to entries in the users's
        password history.

        :arg object user: A :class:`~django.contrib.auth.models.User` instance.
        :arg str raw_password: A unicode string representing a password.
        :returns: ``False`` if a password has been used before, ``True`` if not.
        :rtype: bool
        """
        result = True
        if user.check_password(raw_password):
            result = False
        else:
            entries = self.filter(user=user).all()[:self.default_offset]
            for entry in entries:
                hasher = identify_hasher(entry.password)
                if hasher.verify(raw_password, entry.password):
                    result = False
                    break
        return result
